import dataclasses
from unittest import TestCase

import pytest

from blueair_api import intermediate_representation_aws as ir


class SensorHistoryTest(TestCase):
    def test_history_simple(self):
        sh = ir.SensorHistory([
                {
                    "datapoints": [
                        [
                            "1",
                            "4",
                            "5",
                            "6",
                            "42",
                            None,
                            "51",
                            "27",
                            "87"
                        ],
                        [
                            "3",
                            "1",
                            "2",
                            "3",
                            "43",
                            None,
                            "50",
                            "24",
                            "91"
                        ],
                    ],
                    "sensors": [
                      "pm1",
                      "pm2_5",
                      "pm10",
                      "tVOC",
                      "hcho",
                      "h",
                      "t",
                      "fsp0"
                    ],
                    "start": "1746401990",
                    "end": "1746409190",
                    "did": "1d528642-56a9"
                  }
                ])
        assert sh[0].timestamp == 1
        assert sh[0].values == {
            "pm1": 4,
            "pm2_5": 5,
            "pm10": 6,
            "tVOC": 42,
            "h": 51,
            "t": 27,
            "fsp0": 87
        }
        assert sh[1].timestamp == 3
        assert sh[1].values == {
            "pm1": 1,
            "pm2_5": 2,
            "pm10": 3,
            "tVOC": 43,
            "h": 50,
            "t": 24,
            "fsp0": 91
        }

        assert sh.to_latest().timestamp == 3


class SensorPackTest(TestCase):
  def testSimple(self):
    sp = ir.SensorPack( [
            {'n': 'v', 't': 1, 'v': 1},
            {'n': 'vb', 't': 2, 'vb': True},
            {'n': 'vs', 't': 3, 'vs': "s"},
            {'n': 'vd', 't': 4, 'vd': "MTIzCg=="},  # b'123\n'
            {'n': 'no_t', 'v': 1},
            {'n': 'no_s', 'v': 1},
            {'n': 's', 'v': 1, 's': 2},
            {'n': 'u', 'v': 1, 'u': "unit"},
            {'n': 'no_u', 'v': 1},
     ])

    latest = sp.to_latest()

    assert latest['v'].timestamp == 1
    assert latest['v'].value == 1
    assert isinstance(latest['v'].value, float)

    assert latest['vb'].timestamp == 2
    assert latest['vb'].value is True
    assert isinstance(latest['vb'].value, bool)

    assert latest['vs'].timestamp == 3
    assert latest['vs'].value == "s"
    assert isinstance(latest['vs'].value, str)

    assert latest['vd'].timestamp == 4
    assert latest['vd'].value == b"123\n"
    assert isinstance(latest['vd'].value, bytes)

    assert latest['no_t'].timestamp is None

    assert latest['no_s'].integral is None
    assert latest['s'].integral == 2
    assert latest['s'].value == 1

    assert latest['no_u'].unit is None
    assert latest['u'].unit == "unit"

  def testToLatestMissingT1None(self):
    sp = ir.SensorPack( [
            {'n': 'missing_t', 't': 1, 'v': 1},
            {'n': 'missing_t', 'v': 2},
    ])

    latest = sp.to_latest()
    assert latest['missing_t'].value == 2

  def testToLatestMissingTNoneNone(self):
    sp = ir.SensorPack( [
            {'n': 'missing_t', 'v': 1},
            {'n': 'missing_t', 'v': 2},
    ])

    latest = sp.to_latest()
    assert latest['missing_t'].value == 2

  def testToLatestMissingTNone1(self):
    sp = ir.SensorPack( [
            {'n': 'missing_t', 'v': 1},
            {'n': 'missing_t', 't': 1, 'v': 2},
    ])

    latest = sp.to_latest()
    assert latest['missing_t'].value == 2

  def testToLatestReversedOrder(self):
    sp = ir.SensorPack( [
            {'n': 'missing_t', 't': 2, 'v': 2},
            {'n': 'missing_t', 't': 1, 'v': 1},
    ])

    latest = sp.to_latest()
    assert latest['missing_t'].value == 2


class QueryJsonTest(TestCase):

    def test_mapping_one(self):
        assert ir.query_json({"a": 0}, "a") == 0

    def test_mapping_two(self):
        assert ir.query_json({"a": {"b": 0}}, "a.b") == 0

    def test_sequence(self):
        assert ir.query_json([{"a": 1}], "0.a") == 1

    def test_none(self):
        # last segment not found produces None.
        assert ir.query_json([{"a": 1}], "0.r") is None

    def test_scalar_error(self):
        with pytest.raises(KeyError):
            ir.query_json(3, "0.r")

    def test_key_error(self):
        with pytest.raises(KeyError):
            # intermediate segment not found produces KeyError
            ir.query_json({"a": {"b": 3}}, "r.b")

@dataclasses.dataclass
class FakeObjectType:
    a : str
    extra_fields: ir.MappingType

class ParseJsonTest(TestCase):

    def test_mapping_one(self):
        d = ir.parse_json(FakeObjectType, {
              "one" :{"a": "1", "e" : 1},
              "two" :{"a": "2", "e" : 2},
            })
        assert d == {
                "one" : FakeObjectType(a="1", extra_fields={"e":1}),
                "two" : FakeObjectType(a="2", extra_fields={"e":2}),
            }




from typing import Any

from .callbacks import CallbacksMixin
from .http_blueair import HttpBlueair
from .util import transform_data_points, safely_get_json_value, convert_none_to_not_implemented
from dataclasses import dataclass, field
from logging import getLogger

_LOGGER = getLogger(__name__)


@dataclass(slots=True)
class Device(CallbacksMixin):
    @classmethod
    async def create_device(cls, api, uuid, name, mac, refresh=False):
        _LOGGER.debug("UUID:"+uuid)
        info = await api.get_info(uuid)
        device = Device(
            api=api,
            uuid=uuid,
            name=name,
            mac=mac,
            timezone=info.get("timezone", None),
            compatibility=info.get("compatibility", None),
            model=info.get("model",None),
            firmware=info.get('firmware', None),
            mcu_firmware=info.get("mcuFirmware",None),
            wlan_driver=info.get("wlanDriver",None),
            room_location=info["roomLocation"]
        )
        if refresh:
            await device.refresh()
        _LOGGER.debug(f"create_device blueair device: {device}")
        return device

    api: HttpBlueair = field(repr=False)
    raw_info: dict[str, Any] = field(repr=False, init=False)

    uuid: str | None = None
    name: str | None = None
    timezone: str | None = None
    compatibility: str | None = None
    model: str | None = None
    mac: str | None = None
    firmware: str | None = None
    mcu_firmware: str | None = None
    wlan_driver: str | None = None
    room_location: str | None = None

    brightness: int | None = None
    child_lock: bool | None = None
    night_mode: bool | None = None
    fan_speed: int | None = None
    fan_auto_mode: bool | None = None
    filter_expired: bool | None = None
    wifi_working: bool | None = None

    pm1: int | None = NotImplemented
    pm10: int | None = NotImplemented
    pm25: int | None = NotImplemented
    voc: int | None = NotImplemented
    co2: int | None = NotImplemented
    temperature: float | None = NotImplemented
    humidity: float | None = NotImplemented
    all_pollution: float | None = NotImplemented

    async def refresh(self):
        _LOGGER.debug("Requesting current attributes...")
        self.raw_info = {}
        attributes = await self.api.get_attributes(self.uuid)
        self.raw_info["attributes"] = attributes
        _LOGGER.debug(f"result: {attributes}")
        if "brightness" in attributes:
            self.brightness = int(attributes["brightness"])
        else:
            self.brightness = NotImplemented
        if "child_lock" in attributes:
            self.child_lock = attributes["child_lock"] == "1"
        else:
            self.child_lock = NotImplemented
        if "night_mode" in attributes:
            self.night_mode = attributes["night_mode"] == "1"
        else:
            self.night_mode = NotImplemented
        if "fan_speed" in attributes:
            self.fan_speed = int(attributes["fan_speed"])
        else:
            self.fan_speed = NotImplemented
        if "filter_status" in attributes:
            self.filter_expired = attributes["filter_status"] != "OK"
        else:
            self.filter_expired = NotImplemented
        if "mode" in attributes:
            self.fan_auto_mode = attributes["mode"] == "auto"
        else:
            self.fan_auto_mode = NotImplemented
        if "wifi_status" in attributes:
            self.wifi_working = attributes["wifi_status"] == "1"
        else:
            self.wifi_working = False
        if self.compatibility != "sense+":
            datapoints = transform_data_points(await self.api.get_data_points_since(self.uuid))
            self.raw_info["datapoints"] = datapoints
            for data_point in datapoints:
                _LOGGER.debug(data_point)
                self.pm25 = convert_none_to_not_implemented(safely_get_json_value(data_point, "pm25", int))
                self.pm10 = convert_none_to_not_implemented(safely_get_json_value(data_point, "pm10", int))
                self.pm1 = convert_none_to_not_implemented(safely_get_json_value(data_point, "pm1", int))
                self.voc = convert_none_to_not_implemented(safely_get_json_value(data_point, "voc", int))
                self.co2 = convert_none_to_not_implemented(safely_get_json_value(data_point, "co2", int))
                self.temperature = convert_none_to_not_implemented(safely_get_json_value(data_point, "temperature", int))
                self.humidity = convert_none_to_not_implemented(safely_get_json_value(data_point, "humidity", int))
                self.all_pollution = convert_none_to_not_implemented(safely_get_json_value(data_point, "all_pollution", int))
        _LOGGER.debug(f"refreshed blueair device: {self}")
        self.publish_updates()

    async def set_fan_speed(self, new_speed: str):
        self.fan_speed = int(new_speed)
        await self.api.set_fan_speed(self.uuid, new_speed)
        self.publish_updates()

    async def set_brightness(self, new_brightness: int):
        self.brightness = new_brightness
        await self.api.set_brightness(self.uuid, new_brightness)
        self.publish_updates()

    async def set_child_lock(self, enabled: bool):
        self.child_lock = enabled
        await self.api.set_child_lock(self.uuid, enabled)
        self.publish_updates()

    async def set_fan_auto_mode(self, fan_auto_mode: bool):
        self.fan_auto_mode = fan_auto_mode
        await self.api.set_fan_auto_mode(self.uuid, fan_auto_mode)
        self.publish_updates()




import typing
from typing import Any
from collections.abc import Iterable
import dataclasses
import base64

type ScalarType = str | float | bool | int | None
type MappingType = dict[str, "ObjectType"]
type SequenceType = list["ObjectType"]
type ObjectType = ScalarType | MappingType | SequenceType


def query_json(jsonobj: ObjectType, path: str) -> ObjectType:
    value = jsonobj
    segs = path.split(".")
    for i, seg in enumerate(segs):
        if isinstance(value, list):
            value = value[int(seg)]
        elif isinstance(value, dict):
            if seg in value:
                value = value[seg]
            elif i == len(segs) - 1:
                # last segment returns None if it is not found.
                value = None
            else:
                raise KeyError(
                    f"cannot resolve path segment on a scalar "
                    f"when resolving segment {i}:{seg} of {path}. "
                    f"available keys are {value.keys()}.")
        else:
            raise KeyError(
                f"cannot resolve path segment on a scalar "
                f"when resolving segment {i}:{seg} of {path}.")
    return value

def parse_json[T](kls: type[T], jsonobj: MappingType) -> dict[str, T]:
    """Parses a json mapping object to dict.

    The key is preserved. The value is parsed as dataclass type kls.
    """
    assert dataclasses.is_dataclass(kls)
    result = {}
    fields = dataclasses.fields(kls)

    for key, value in jsonobj.items():
        if not isinstance(value, dict):
            raise TypeError("expecting mapping value to be dict.")
        extra_fields = dict(value)  # make extra_fields copy.
        kwargs : dict[str, Any] = {}
        for field in fields:
            if field.name == "extra_fields":
                kwargs[field.name] = extra_fields
            elif field.default is dataclasses.MISSING:
                kwargs[field.name] = extra_fields.pop(field.name)
            else:
                kwargs[field.name] = extra_fields.pop(field.name, field.default)

        obj = kls(**kwargs)
        result[key] = typing.cast(T, obj)
    return result


########################
# Blueair AWS API Schema.

@dataclasses.dataclass
class Attribute:
    """DeviceAttribute(da); defines an attribute

    An attribute is most likely mutable. An attribute may
    also have alias names, likely derived from the 'dc' relation
    e.g. a/sb, a/standby all refer to the 'sb' attribute.
    """
    extra_fields : MappingType
    n: str   # name
    a: int | bool   # default attribute value, example value?
    e: bool     # ??? always True
    fe:bool     # ???  always True
    ot: str     # object type? topic type?
    p: bool     # only false for reboot and sflu
    tn: str   # topic name a path-like name d/????/a/{n}


@dataclasses.dataclass
class Sensor:
    """DeviceSensor(ds); seems to define a sensor.

    We never directly access these objects. Though this defines
    the schema for 'h', 't', 'pm10' etc that gets returned in
    the sensor_data senml SensorPack.
    """
    extra_fields : MappingType
    n: str    # name
    i: int    # integration time? in millis
    e: bool   # ???
    fe: bool  # ??? always True.
    ot: str   # object type / topic name
    tn: str   # topic name a path-like name d/????/s/{n}
    ttl: int  # only seen 0 or -1, not sure if used.
    tf: str | None = None   # senml+json; topic format

@dataclasses.dataclass
class Control:
    """DeviceControl (dc); seems to define a state.

    The states SensorPack seem to be using fields defined
    in dc. The only exception is 'online' which is not defined
    here.
    """
    extra_fields : MappingType
    n: str  # name
    v: int | bool
    a: str | None = None
    s: str | None = None
    d: str | None = None  # device info json path


@dataclasses.dataclass
class SensorRecord:
    values: dict[str, int]
    timestamp: float | None


class SensorHistory(list[SensorRecord]):
    def __init__(self, response):
        if response is None:
            sensors = []
            datapoints = []
        else:
            sensors = response[0]["sensors"]
            datapoints = response[0]["datapoints"]
        sensor_records = []
        for datapoint in datapoints:
            values = {}
            for idx, sensor in enumerate(sensors):
                if datapoint[idx+1] is not None:
                    values[sensor] = int(datapoint[idx+1])
            sensor_records.append(SensorRecord(timestamp=int(datapoint[0]), values=values))
        super().__init__(sensor_records)

    def to_latest(self) -> SensorRecord:
        def key(e):
            return e.timestamp
        self.sort(key=key, reverse=True)
        if len(self) > 0:
            return self[0]
        else:
            return SensorRecord(values={},  timestamp=0)



########################
# SenML RFC8428

@dataclasses.dataclass
class Record:
    """A RFC8428 SenML record, resolved to Python types."""
    name: str
    unit: str | None
    value: float | bool | str | bytes
    timestamp: float | None
    integral: float | None


class SensorPack(list[Record]):
    """Represents a RFC8428 SensorPack, resolved to Python Types."""

    def __init__(self, stream: Iterable[MappingType]):
        seq = []
        for record in stream:
            rs = None
            rt = None
            rn : str
            ru = None
            rv : float | bool | str | bytes
            for label, value in record.items():
                assert isinstance(value, str | int | float | bool)
                match label:
                    case 'bn' | 'bt' | 'bu' | 'bv' | 'bs' | 'bver':
                        raise ValueError("TODO: base fields not supported. c.f. RFC8428, 4.1")
                    case 't':
                        rt = float(value)
                    case 's':
                        rs = float(value)
                    case 'v':
                        rv = float(value)
                    case 'vb':
                        rv = bool(value)
                    case 'vs':
                        rv = str(value)
                    case 'vd':
                        rv = bytes(base64.b64decode(str(value)))
                    case 'n':
                        rn = str(value)
                    case 'u':
                        ru = str(value)
            seq.append(Record(name=rn, unit=ru, value=rv, integral=rs, timestamp=rt))
        super().__init__(seq)

    def to_latest_value(self) -> dict[str, str | bool | float | bytes]:
        return {rn : record.value for rn, record in self.to_latest().items()}

    def to_latest(self) -> dict[str, Record]:
        latest : dict[str, Record] = {}
        for record in self:
            rn = record.name
            if record.name not in latest:
                latest[rn] = record
                continue
            lt = latest[record.name].timestamp
            if record.timestamp is None:
                latest[rn] = record
            elif lt is None:
                latest[rn] = record
            elif lt < record.timestamp:
                latest[rn] = record
        return latest



from aiohttp import ClientError


class BaseError(ClientError):
    pass


class RateError(BaseError):
    pass


class AuthError(BaseError):
    pass


class LoginError(AuthError):
    pass


class SessionError(AuthError):
    pass



from .errors import BaseError, RateError, AuthError, SessionError, LoginError
from .http_blueair import HttpBlueair
from .http_aws_blueair import HttpAwsBlueair
from .util_bootstrap import get_devices, get_aws_devices
from .device import Device
from .device_aws import DeviceAws
from .model_enum import ModelEnum



from typing import Any

from logging import getLogger
from json import dumps

from .callbacks import CallbacksMixin
from .http_aws_blueair import HttpAwsBlueair
from .model_enum import ModelEnum
from . import intermediate_representation_aws as ir
from dataclasses import dataclass, field

_LOGGER = getLogger(__name__)

type AttributeType[T] = T | None

@dataclass(slots=True)
class DeviceAws(CallbacksMixin):
    @classmethod
    async def create_device(cls, api, uuid, name, mac, type_name, refresh=False):
        _LOGGER.debug("UUID:"+uuid)
        device_aws = DeviceAws(
            api=api,
            uuid=uuid,
            name_api=name,
            mac=mac,
            type_name=type_name,
        )
        if refresh:
            await device_aws.refresh()
        _LOGGER.debug(f"create_device blueair device_aws: {device_aws}")
        return device_aws

    api: HttpAwsBlueair = field(repr=False)
    raw_info : dict[str, Any] = field(repr=False, init=False)
    raw_sensors : dict[str, Any] = field(repr=False, init=False)

    uuid : str | None = None
    name : str | None = None
    name_api : str | None = None
    mac : str | None = None
    type_name : str | None = None

    # Attributes are defined below.
    # We mandate that unittests shall test all fields of AttributeType.
    sku : AttributeType[str] = None
    firmware : AttributeType[str] = None
    mcu_firmware : AttributeType[str] = None
    serial_number : AttributeType[str] = None

    brightness : AttributeType[int] = None
    child_lock : AttributeType[bool] = None
    fan_speed : AttributeType[int] = None
    fan_auto_mode : AttributeType[bool] = None
    standby : AttributeType[bool] = None
    night_mode : AttributeType[bool] = None
    germ_shield : AttributeType[bool] = None

    pm1 : AttributeType[int] = None
    pm2_5 : AttributeType[int] = None
    pm10 : AttributeType[int] = None
    total_voc : AttributeType[int] = None
    voc : AttributeType[int] = None
    temperature : AttributeType[int] = None
    humidity : AttributeType[int] = None
    filter_usage_percentage : AttributeType[int] = None
    wifi_working : AttributeType[bool] = None

    wick_usage_percentage : AttributeType[int] = None
    wick_dry_mode : AttributeType[bool] = None
    water_shortage : AttributeType[bool] = None
    auto_regulated_humidity : AttributeType[int] = None

    main_mode: AttributeType[int] = None # api value 0 purify only, 1 heat on, 2 cool on
    heat_sub_mode: AttributeType[int] = None # api value 1 heat on, 2 heat on with fan auto
    heat_fan_speed: AttributeType[int] = None # api value 11/37/64/91
    heat_temp: AttributeType[int] = None # api value is celsius * 10
    cool_sub_mode: AttributeType[int] = None # api value 1 cool on, 2 cool on with fan auto
    cool_fan_speed: AttributeType[int] = None # api value 11/37/64/91
    ap_sub_mode: AttributeType[int] = None # api value 1 manual speeed, 2 auto fan speed
    fan_speed_0: AttributeType[int] = None # api value 11/37/64/91
    temperature_unit: AttributeType[int] = None # api value of 1 is celcius

    async def refresh(self):
        _LOGGER.debug(f"refreshing blueair device aws: {self}")
        self.raw_info = await self.api.device_info(self.name_api, self.uuid)
        self.raw_sensors = await self.api.device_sensors(self.name_api, self.uuid)
        _LOGGER.debug(dumps(self.raw_info, indent=2))
        if self.raw_sensors is not None:
            _LOGGER.debug(dumps(self.raw_sensors, indent=2))

        ds = ir.parse_json(ir.Sensor, ir.query_json(self.raw_info, "configuration.ds"))
        dc = ir.parse_json(ir.Control, ir.query_json(self.raw_info, "configuration.dc"))

        sensor_data = ir.SensorHistory(self.raw_sensors).to_latest()

        def sensor_data_safe_get(key):
            return sensor_data.values.get(key) if key in ds else NotImplemented

        self.pm1 = sensor_data_safe_get("pm1")
        self.pm2_5 = sensor_data_safe_get("pm2_5")
        self.pm10 = sensor_data_safe_get("pm10")
        self.total_voc = sensor_data_safe_get("tVOC")
        self.voc = sensor_data_safe_get("voc")
        self.temperature = sensor_data_safe_get("t")
        self.humidity = sensor_data_safe_get("h")
        self.fan_speed_0 = sensor_data_safe_get("fsp0")

        def info_safe_get(path):
            # directly reads for the schema. If the schema field is
            # undefined, it is NotImplemented, not merely unavailable.
            value = ir.query_json(self.raw_info, path)
            if value is None:
                return NotImplemented
            return value

        self.name = info_safe_get("configuration.di.name")
        self.firmware = info_safe_get("configuration.di.cfv")
        self.mcu_firmware = info_safe_get("configuration.di.mfv")
        self.serial_number = info_safe_get("configuration.di.ds")
        self.sku = info_safe_get("configuration.di.sku")

        states = ir.SensorPack(self.raw_info["states"]).to_latest_value()

        def states_safe_get(key):
            return states.get(key) if key in dc else NotImplemented

        # "online" is not defined in the schema.
        self.wifi_working = states.get("online")

        self.standby = states_safe_get("standby")
        self.night_mode = states_safe_get("nightmode")
        self.germ_shield = states_safe_get("germshield")
        self.brightness = states_safe_get("brightness")
        self.child_lock = states_safe_get("childlock")
        self.fan_speed = states_safe_get("fanspeed")
        self.fan_auto_mode = states_safe_get("automode")
        self.filter_usage_percentage = states_safe_get("filterusage")

        self.wick_usage_percentage = states_safe_get("wickusage")
        self.wick_dry_mode = states_safe_get("wickdrys")
        self.auto_regulated_humidity = states_safe_get("autorh")
        self.water_shortage = states_safe_get("wshortage")

        self.main_mode = states_safe_get("mainmode")
        self.heat_temp = states_safe_get("heattemp")
        self.heat_sub_mode = states_safe_get("heatsubmode")
        self.heat_fan_speed = states_safe_get("heatfs")
        self.cool_sub_mode = states_safe_get("coolsubmode")
        self.cool_fan_speed = states_safe_get("coolfs")
        self.ap_sub_mode = states_safe_get("apsubmode")
        if states_safe_get("fsp0") is NotImplemented:
            self.fan_speed_0 = sensor_data_safe_get("fsp0")
        else:
            self.fan_speed_0 = states_safe_get("fsp0")
        self.temperature_unit = states_safe_get("tu")

        self.publish_updates()
        _LOGGER.debug(f"refreshed blueair device aws: {self}")

    async def set_brightness(self, value: int):
        self.brightness = value
        await self.api.set_device_info(self.uuid, "brightness", "v", value)
        self.publish_updates()

    async def set_fan_speed(self, value: int):
        self.fan_speed = value
        await self.api.set_device_info(self.uuid, "fanspeed", "v", value)
        self.publish_updates()

    async def set_standby(self, value: bool):
        self.standby = value
        await self.api.set_device_info(self.uuid, "standby", "vb", value)
        self.publish_updates()

    async def set_fan_auto_mode(self, fan_auto_mode: bool):
        self.fan_auto_mode = fan_auto_mode
        await self.api.set_device_info(self.uuid, "automode", "vb", fan_auto_mode)
        self.publish_updates()

    async def set_auto_regulated_humidity(self, value: int):
        self.auto_regulated_humidity = value
        await self.api.set_device_info(self.uuid, "autorh", "v", value)
        self.publish_updates()

    async def set_child_lock(self, child_lock: bool):
        self.child_lock = child_lock
        await self.api.set_device_info(self.uuid, "childlock", "vb", child_lock)
        self.publish_updates()

    async def set_night_mode(self, night_mode: bool):
        self.night_mode = night_mode
        await self.api.set_device_info(self.uuid, "nightmode", "vb", night_mode)
        self.publish_updates()

    async def set_wick_dry_mode(self, value: bool):
        self.wick_dry_mode = value
        await self.api.set_device_info(self.uuid, "wickdrys", "vb", value)
        self.publish_updates()

    async def set_germ_shield(self, value: bool):
        self.germ_shield = value
        await self.api.set_device_info(self.uuid, "germshield", "vb", value)
        self.publish_updates()

    async def set_main_mode(self, value: int):
        self.main_mode = value
        await self.api.set_device_info(self.uuid, "mainmode", "v", value)
        self.publish_updates()

    async def set_heat_temp(self, value: int):
        self.heat_temp = value
        await self.api.set_device_info(self.uuid, "heattemp", "v", value)
        self.publish_updates()

    async def set_heat_sub_mode(self, value: int):
        self.heat_sub_mode = value
        await self.api.set_device_info(self.uuid, "heatsubmode", "v", value)
        self.publish_updates()

    async def set_heat_fan_speed(self, value: int):
        self.heat_fan_speed = value
        await self.api.set_device_info(self.uuid, "heatfs", "v", value)
        self.publish_updates()

    async def set_cool_sub_mode(self, value: int):
        self.cool_sub_mode = value
        await self.api.set_device_info(self.uuid, "coolsubmode", "v", value)
        self.publish_updates()

    async def set_cool_fan_speed(self, value: int):
        self.cool_fan_speed = value
        await self.api.set_device_info(self.uuid, "coolfs", "v", value)
        self.publish_updates()

    async def set_ap_sub_mode(self, value: int):
        self.ap_sub_mode = value
        await self.api.set_device_info(self.uuid, "apsubmode", "v", value)
        self.publish_updates()

    async def set_fan_speed_0(self, value: int):
        self.fan_speed_0 = value
        await self.api.set_device_info(self.uuid, "fsp0", "v", value)
        self.publish_updates()

    @property
    def model(self) -> ModelEnum:
        if self.sku == "111633":
            return ModelEnum.HUMIDIFIER_H35I
        if self.sku == "105820":
            return ModelEnum.PROTECT_7440I
        if self.sku == "105826":
            return ModelEnum.PROTECT_7470I
        if self.sku == "110059":
            return ModelEnum.MAX_211I
        if self.sku in ["110092", "110829"]:
            return ModelEnum.MAX_311I
        if self.sku == "110057":
            return ModelEnum.MAX_411I
        if self.sku in ["112637", "111178"]:
            return ModelEnum.MAX_511I
        if self.sku == "112124":
            return ModelEnum.T10I
        if self.sku in ["109539", "110034"]:
            return ModelEnum.MAX_311I_PLUS
        if self.sku == "112929":
            return ModelEnum.BLUE_SIGNATURE
        if self.sku == "112793":
            return ModelEnum.PET_AIR_PRO
        if self.sku == "110178":
            return ModelEnum.MAX_3250I
        if self.sku == "111905":
            return ModelEnum.MAX_3650I
        return ModelEnum.UNKNOWN



from enum import StrEnum


class ModelEnum(StrEnum):
    UNKNOWN = "Unknown"
    HUMIDIFIER_H35I = "Blueair Humidifier H35i"
    PROTECT_7440I = "Blueair Protect 7440i"
    PROTECT_7470I = "Blueair Protect 7470i"
    MAX_211I = "Blueair Blue Pure 211i Max"
    MAX_311I = "Blueair Blue Pure 311i Max"
    MAX_311I_PLUS = "Blueair Blue Pure 311i+ Max"
    MAX_3250I = "Blueair Blue Pure 3250i Max"
    MAX_3650I = "Blueair Blue Pure 3650i Max"
    MAX_411I = "Blueair Blue Pure 411i Max"
    MAX_511I = "Blueair Blue Pure 511i Max"
    T10I = "T10i ComfortPure 3-in-1 Filter/Heater/Fan"
    BLUE_SIGNATURE = "Blue Signature"
    PET_AIR_PRO = "PetAir Pro"



"""Tests for DeviceAws.

Here is one way to run it:

First install the package in developer mode

    $ pip install -e .

Then use pytest to drive the tests

    $ pytest tests
"""
import typing
from typing import Any

import contextlib
import dataclasses
from importlib import resources
import json
from unittest import mock
from unittest import IsolatedAsyncioTestCase

import pytest

from blueair_api.device_aws import DeviceAws, AttributeType
from blueair_api.model_enum import ModelEnum
from blueair_api import http_aws_blueair
from blueair_api import intermediate_representation_aws as ir


class FakeDeviceInfoHelper:
    """Fake for the 'device info' interface of HttpAwsBlueAir class."""
    def __init__(self, info: dict[str, Any]):
        self.info = info

    async def device_info(self, *args, **kwargs):
        return self.info

    async def set_device_info(self, device_uuid, service_name, action_verb, action_value):
        # this function seems to be only updating the states consider rename the method.
        # action_verb seems to be a type annotation:
        # c.f. senml: https://www.rfc-editor.org/rfc/rfc8428.html#section-5
        # the senml parsing library (utils.py) could use some additional love.
        # to make it more conformal to the RFC standard.
        for state in self.info['states']:
            if state['n'] == service_name:
                break
        else:
            state = {'n': service_name}
            self.info['states'].append(state)
        # Watch out: mutate after append produces desired mutation to info.
        state[action_verb] = action_value


class AssertFullyCheckedHelper:
    """Assert that all attributes of AttributeType are accessed once."""
    def __init__(self, device : DeviceAws):
        self.device = device
        self.logs = []
        self.fields = set()
        for field in dataclasses.fields(self.device):
            if typing.get_origin(field.type) is AttributeType:
                self.fields.add(field.name)

    def __getattr__(self, attr):
        if attr in self.fields:
            self.logs.append(attr)
        return getattr(self.device, attr)


@contextlib.contextmanager
def assert_fully_checked(device):
    helper = AssertFullyCheckedHelper(device)
    yield helper
    assert set(helper.logs) == helper.fields


class DeviceAwsTestBase(IsolatedAsyncioTestCase):

    def setUp(self):

        patcher = mock.patch('blueair_api.http_aws_blueair.HttpAwsBlueair', autospec=True)
        self.api_class = patcher.start()
        self.addCleanup(patcher.stop)
        self.api = self.api_class(username="fake-username", password="fake-password")

        self.device = DeviceAws(self.api,
             name_api="fake-name-api",
             uuid="fake-uuid",
             name="fake-name",
             mac="fake-mac",
             type_name='fake-type-name')

        self.device_info_helper = FakeDeviceInfoHelper(
           {"configuration": {"di" : {}, "ds" : {}, "dc" : {}, "da" : {},},
            "sensordata": [],
            "states": [],
           })

        self.device_sensor_helper = {"mock_data": [{
                "datapoints": [],
                "sensors": []
            }]}
        async def fake_sensors(device_name, device_uuid):
            return self.device_sensor_helper["mock_data"]

        self.api.device_sensors.side_effect = fake_sensors
        self.api.device_info.side_effect = self.device_info_helper.device_info
        self.api.set_device_info.side_effect = self.device_info_helper.set_device_info


class DeviceAwsSetterTest(DeviceAwsTestBase):
    """Tests for all of the setters."""

    def setUp(self):
        super().setUp()
        # minimally populate dc to define the states.
        fake = {"n": "n", "v": 0}
        ir.query_json(self.device_info_helper.info, "configuration.dc").update({
            "brightness": fake,
            "fanspeed": fake,
            "standby": fake,
            "germshield": fake,
            "automode": fake,
            "autorh": fake,
            "childlock": fake,
            "nightmode": fake,
            "wickdrys": fake,
            "mainmode": fake,
            "heattemp": fake,
            "heatsubmode": fake,
            "heatfs": fake,
            "coolsubmode": fake,
            "coolfs": fake,
            "apsubmode": fake,
            "fsp0": fake,
        })

    async def test_brightness(self):
        # test cache works
        self.device.brightness = None
        await self.device.set_brightness(1)
        assert self.device.brightness == 1

        # test refresh works
        await self.device.set_brightness(2)
        self.device.brightness = None
        await self.device.refresh()
        assert self.device.brightness == 2

    async def test_fan_speed(self):
        # test cache works
        self.device.fan_speed = None
        await self.device.set_fan_speed(1)
        assert self.device.fan_speed == 1

        # test refresh works
        await self.device.set_fan_speed(2)
        self.device.fan_speed = None
        await self.device.refresh()
        assert self.device.fan_speed == 2

    async def test_germ_shield(self):
        # test cache works
        self.device.germ_shield = None
        await self.device.set_germ_shield(False)
        assert self.device.germ_shield is False

        # test refresh works
        await self.device.set_germ_shield(True)
        self.device.germ_shield = None
        await self.device.refresh()
        assert self.device.germ_shield is True

    async def test_standby(self):
        # test cache works
        self.device.standby = None
        await self.device.set_standby(False)
        assert self.device.standby is False

        # test refresh works
        await self.device.set_standby(True)
        self.device.standby = None
        await self.device.refresh()
        assert self.device.standby is True

    async def test_fan_auto_mode(self):
        # test cache works
        self.device.fan_auto_mode = None
        await self.device.set_fan_auto_mode(False)
        assert self.device.fan_auto_mode is False

        # test refresh works
        await self.device.set_fan_auto_mode(True)
        self.device.fan_auto_mode = None
        await self.device.refresh()
        assert self.device.fan_auto_mode is True

    async def test_auto_regulated_humidity(self):
        # test cache works
        self.device.auto_regulated_humidity = None
        await self.device.set_auto_regulated_humidity(1)
        assert self.device.auto_regulated_humidity == 1

        # test refresh works
        await self.device.set_auto_regulated_humidity(2)
        self.device.auto_regulated_humidity = None
        await self.device.refresh()
        assert self.device.auto_regulated_humidity == 2

    async def test_child_lock(self):
        # test cache works
        self.device.child_lock = None
        await self.device.set_child_lock(False)
        assert self.device.child_lock is False

        # test refresh works
        await self.device.set_child_lock(True)
        self.device.child_lock = None
        await self.device.refresh()
        assert self.device.child_lock is True

    async def test_night_mode(self):
        # test cache works
        self.device.night_mode = None
        await self.device.set_night_mode(False)
        assert self.device.night_mode is False

        # test refresh works
        await self.device.set_night_mode(True)
        self.device.night_mode = None
        await self.device.refresh()
        assert self.device.night_mode is True

    async def test_wick_dry_mode(self):
        # test cache works
        self.device.wick_dry_mode = None
        await self.device.set_wick_dry_mode(False)
        assert self.device.wick_dry_mode is False

        # test refresh works
        await self.device.set_wick_dry_mode(True)
        self.device.wick_dry_mode = None
        await self.device.refresh()
        assert self.device.wick_dry_mode is True

    async def test_main_mode(self):
        # test cache works
        self.device.main_mode = None
        await self.device.set_main_mode(1)
        assert self.device.main_mode == 1

        # test refresh works
        await self.device.set_main_mode(2)
        self.device.main_mode = None
        await self.device.refresh()
        assert self.device.main_mode == 2

    async def test_heat_temp(self):
        # test cache works
        self.device.heat_temp = None
        await self.device.set_heat_temp(1)
        assert self.device.heat_temp == 1

        # test refresh works
        await self.device.set_heat_temp(2)
        self.device.heat_temp = None
        await self.device.refresh()
        assert self.device.heat_temp == 2

    async def test_heat_sub_mode(self):
        # test cache works
        self.device.heat_sub_mode = None
        await self.device.set_heat_sub_mode(1)
        assert self.device.heat_sub_mode == 1

        # test refresh works
        await self.device.set_heat_sub_mode(2)
        self.device.heat_sub_mode = None
        await self.device.refresh()
        assert self.device.heat_sub_mode == 2

    async def test_heat_fan_speed(self):
        # test cache works
        self.device.heat_fan_speed = None
        await self.device.set_heat_fan_speed(1)
        assert self.device.heat_fan_speed == 1

        # test refresh works
        await self.device.set_heat_fan_speed(2)
        self.device.heat_fan_speed = None
        await self.device.refresh()
        assert self.device.heat_fan_speed == 2

    async def test_cool_sub_mode(self):
        # test cache works
        self.device.cool_sub_mode = None
        await self.device.set_cool_sub_mode(1)
        assert self.device.cool_sub_mode == 1

        # test refresh works
        await self.device.set_cool_sub_mode(2)
        self.device.cool_sub_mode = None
        await self.device.refresh()
        assert self.device.cool_sub_mode == 2

    async def test_cool_fan_speed(self):
        # test cache works
        self.device.cool_fan_speed = None
        await self.device.set_cool_fan_speed(1)
        assert self.device.cool_fan_speed == 1

        # test refresh works
        await self.device.set_cool_fan_speed(2)
        self.device.cool_fan_speed = None
        await self.device.refresh()
        assert self.device.cool_fan_speed == 2

    async def test_ap_sub_mode(self):
        # test cache works
        self.device.ap_sub_mode = None
        await self.device.set_ap_sub_mode(1)
        assert self.device.ap_sub_mode == 1

        # test refresh works
        await self.device.set_ap_sub_mode(2)
        self.device.ap_sub_mode = None
        await self.device.refresh()
        assert self.device.ap_sub_mode == 2

    async def test_fan_speed_0(self):
        # test cache works
        self.device.fan_speed_0 = None
        await self.device.set_fan_speed_0(1)
        assert self.device.fan_speed_0 == 1

        # test refresh works
        await self.device.set_fan_speed_0(2)
        self.device.fan_speed_0 = None
        await self.device.refresh()
        assert self.device.fan_speed_0 == 2


class EmptyDeviceAwsTest(DeviceAwsTestBase):
    """Tests for a emptydevice.

    This is a made-up device. All attrs are not implemented.

    Other device types shall override setUp and populate self.info with the
    golden dataset.
    """

    async def test_attributes(self):

        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.UNKNOWN

            assert device.pm1 is NotImplemented
            assert device.pm2_5 is NotImplemented
            assert device.pm10 is NotImplemented
            assert device.total_voc is NotImplemented
            assert device.voc is NotImplemented
            assert device.temperature is NotImplemented
            assert device.humidity is NotImplemented
            assert device.name is NotImplemented
            assert device.firmware is NotImplemented
            assert device.mcu_firmware is NotImplemented
            assert device.serial_number is NotImplemented
            assert device.sku is NotImplemented

            assert device.standby is NotImplemented
            assert device.night_mode is NotImplemented
            assert device.germ_shield is NotImplemented
            assert device.brightness is NotImplemented
            assert device.child_lock is NotImplemented
            assert device.fan_speed is NotImplemented
            assert device.fan_auto_mode is NotImplemented
            assert device.filter_usage_percentage is NotImplemented
            assert device.wifi_working is None
            assert device.wick_usage_percentage is NotImplemented
            assert device.auto_regulated_humidity is NotImplemented
            assert device.water_shortage is NotImplemented
            assert device.wick_dry_mode is NotImplemented
            assert device.main_mode is NotImplemented
            assert device.ap_sub_mode is NotImplemented
            assert device.heat_temp is NotImplemented
            assert device.heat_sub_mode is NotImplemented
            assert device.heat_fan_speed is NotImplemented
            assert device.cool_sub_mode is NotImplemented
            assert device.cool_fan_speed is NotImplemented
            assert device.fan_speed_0 is NotImplemented
            assert device.temperature_unit is NotImplemented


class H35iTest(DeviceAwsTestBase):
    """Tests for H35i."""

    def setUp(self):
        super().setUp()
        with open(resources.files().joinpath('device_info/H35i.json')) as sample_file:
            info = json.load(sample_file)
        self.device_info_helper.info.update(info)

    async def test_attributes(self):

        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.HUMIDIFIER_H35I

            assert device.pm1 is NotImplemented
            assert device.pm2_5 is NotImplemented
            assert device.pm10 is NotImplemented
            assert device.total_voc is NotImplemented
            assert device.voc is NotImplemented
            assert device.temperature is None
            assert device.humidity is None
            assert device.name == "Bedroom"
            assert device.firmware == "1.0.1"
            assert device.mcu_firmware == "1.0.1"
            assert device.serial_number == "111163300201110210004036"
            assert device.sku == "111633"

            assert device.standby is False
            assert device.night_mode is False
            assert device.germ_shield is NotImplemented
            assert device.brightness == 49
            assert device.child_lock is False
            assert device.fan_speed == 24
            assert device.fan_auto_mode is False
            assert device.filter_usage_percentage is NotImplemented
            assert device.wifi_working is True
            assert device.wick_usage_percentage == 13
            assert device.auto_regulated_humidity == 50
            assert device.water_shortage is False
            assert device.wick_dry_mode is False
            assert device.main_mode is NotImplemented
            assert device.ap_sub_mode is NotImplemented
            assert device.heat_temp is NotImplemented
            assert device.heat_sub_mode is NotImplemented
            assert device.heat_fan_speed is NotImplemented
            assert device.cool_sub_mode is NotImplemented
            assert device.cool_fan_speed is NotImplemented
            assert device.fan_speed_0 is None
            assert device.temperature_unit is NotImplemented


class Max311iTest(DeviceAwsTestBase):
    """Tests for H35i."""

    def setUp(self):
        super().setUp()
        with open(resources.files().joinpath('device_info/max_311i.json')) as sample_file:
            info = json.load(sample_file)
        self.device_info_helper.info.update(info)

    async def test_attributes(self):
        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.MAX_311I

            assert device.pm1 is NotImplemented
            assert device.pm2_5 is None
            assert device.pm10 is NotImplemented
            assert device.total_voc is NotImplemented
            assert device.voc is NotImplemented
            assert device.temperature is NotImplemented
            assert device.humidity is NotImplemented
            assert device.name == "Loft"
            assert device.firmware == "1.0.4"
            assert device.mcu_firmware == "1.0.4"
            assert device.serial_number == "111082900302313210005018"
            assert device.sku == "110829"

            assert device.standby is False
            assert device.night_mode is False
            assert device.germ_shield is NotImplemented
            assert device.brightness == 35
            assert device.child_lock is True
            assert device.fan_speed == 11
            assert device.fan_auto_mode is True
            assert device.filter_usage_percentage == 12
            assert device.wifi_working is True
            assert device.wick_usage_percentage is NotImplemented
            assert device.auto_regulated_humidity is NotImplemented
            assert device.water_shortage is NotImplemented
            assert device.wick_dry_mode is NotImplemented
            assert device.main_mode is NotImplemented
            assert device.ap_sub_mode is NotImplemented
            assert device.heat_temp is NotImplemented
            assert device.heat_sub_mode is NotImplemented
            assert device.heat_fan_speed is NotImplemented
            assert device.cool_sub_mode is NotImplemented
            assert device.cool_fan_speed is NotImplemented
            assert device.fan_speed_0 is None
            assert device.temperature_unit is NotImplemented


class T10iTest(DeviceAwsTestBase):
    """Tests for T10i."""

    def setUp(self):
        super().setUp()
        with open(resources.files().joinpath('device_info/T10i.json')) as sample_file:
            info = json.load(sample_file)
        self.device_info_helper.info.update(info)

    async def test_attributes(self):

        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.T10I

            assert device.pm1 is NotImplemented
            assert device.pm2_5 is None
            assert device.pm10 is NotImplemented
            assert device.total_voc is NotImplemented
            assert device.voc is NotImplemented
            assert device.temperature is None
            assert device.humidity is None
            assert device.name == "Allen's Office"
            assert device.firmware == "1.0.4"
            assert device.mcu_firmware == "1.0.4"
            assert device.serial_number == "111212400002313210001961"
            assert device.sku == "112124"

            assert device.standby is False
            assert device.night_mode is NotImplemented
            assert device.germ_shield is NotImplemented
            assert device.brightness == 100
            assert device.child_lock is False
            assert device.fan_speed is NotImplemented
            assert device.fan_auto_mode is NotImplemented
            assert device.filter_usage_percentage == 0
            assert device.wifi_working is True
            assert device.wick_usage_percentage is NotImplemented
            assert device.auto_regulated_humidity is NotImplemented
            assert device.water_shortage is NotImplemented
            assert device.wick_dry_mode is NotImplemented
            assert device.main_mode == 1
            assert device.heat_temp == 230
            assert device.heat_sub_mode == 2
            assert device.heat_fan_speed == 11
            assert device.cool_sub_mode == 1
            assert device.cool_fan_speed == 11
            assert device.ap_sub_mode == 1
            assert device.fan_speed_0 == 11
            assert device.temperature_unit == 1


class Protect7470iTest(DeviceAwsTestBase):
    """Tests for protect7470i."""

    def setUp(self):
        super().setUp()
        with open(resources.files().joinpath('device_info/protect_7470i.json')) as sample_file:
            info = json.load(sample_file)
        self.device_info_helper.info.update(info)
        with open(resources.files().joinpath('device_info/protect_7470i_sensors.json')) as sample_file:
            sensors = json.load(sample_file)
        self.device_sensor_helper["mock_data"] = sensors

    async def test_attributes(self):

        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.PROTECT_7470I
            assert device.pm1 == 0
            assert device.pm2_5 == 0
            assert device.pm10 == 0
            assert device.total_voc == 134
            assert device.voc is NotImplemented
            assert device.temperature == 23
            assert device.humidity == 55
            assert device.name == "air filter in room"
            assert device.firmware == "2.1.1"
            assert device.mcu_firmware == "1.0.12"
            assert device.serial_number == "110582600000110110016855"
            assert device.sku == "105826"

            assert device.standby is False
            assert device.night_mode is False
            assert device.germ_shield is True
            assert device.brightness == 100
            assert device.child_lock is True
            assert device.fan_speed == 91
            assert device.fan_auto_mode is False
            assert device.filter_usage_percentage == 50
            assert device.wifi_working is True
            assert device.wick_usage_percentage is NotImplemented
            assert device.auto_regulated_humidity is NotImplemented
            assert device.water_shortage is NotImplemented
            assert device.wick_dry_mode is NotImplemented
            assert device.main_mode is NotImplemented
            assert device.heat_temp is NotImplemented
            assert device.heat_sub_mode is NotImplemented
            assert device.heat_fan_speed is NotImplemented
            assert device.cool_sub_mode is NotImplemented
            assert device.cool_fan_speed is NotImplemented
            assert device.ap_sub_mode is NotImplemented
            assert device.fan_speed_0 == 91
            assert device.temperature_unit is NotImplemented


class Max211iTest(DeviceAwsTestBase):
    """Tests for Max 211i."""

    def setUp(self):
        super().setUp()
        with open(resources.files().joinpath('device_info/max_211i.json')) as sample_file:
            info = json.load(sample_file)
        self.device_info_helper.info.update(info)

    async def test_attributes(self):

        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.MAX_211I

            assert device.pm1 is None
            assert device.pm2_5 is None
            assert device.pm10 is None
            assert device.total_voc is NotImplemented
            assert device.voc is NotImplemented
            assert device.temperature is NotImplemented
            assert device.humidity is NotImplemented
            assert device.name == "Bedroom Purifier"
            assert device.firmware == "1.1.6"
            assert device.mcu_firmware == "1.1.6"
            assert device.serial_number == "111005900201111210085956"
            assert device.sku == "110059"

            assert device.standby is False
            assert device.night_mode is False
            assert device.germ_shield is NotImplemented
            assert device.brightness == 0
            assert device.child_lock is False
            assert device.fan_speed == 15
            assert device.fan_auto_mode is False
            assert device.filter_usage_percentage == 55
            assert device.wifi_working is True
            assert device.wick_usage_percentage is NotImplemented
            assert device.auto_regulated_humidity is NotImplemented
            assert device.water_shortage is NotImplemented
            assert device.wick_dry_mode is NotImplemented
            assert device.main_mode is NotImplemented
            assert device.heat_temp is NotImplemented
            assert device.heat_sub_mode is NotImplemented
            assert device.heat_fan_speed is NotImplemented
            assert device.cool_sub_mode is NotImplemented
            assert device.cool_fan_speed is NotImplemented
            assert device.ap_sub_mode is NotImplemented
            assert device.fan_speed_0 is None
            assert device.temperature_unit is NotImplemented


class PetAirProTest(DeviceAwsTestBase):
    """Tests for PetPro."""

    def setUp(self):
        super().setUp()
        with open(resources.files().joinpath('device_info/pet_air_pro.json')) as sample_file:
            info = json.load(sample_file)
        self.device_info_helper.info.update(info)

    async def test_attributes(self):

        await self.device.refresh()
        self.api.device_info.assert_awaited_with("fake-name-api", "fake-uuid")

        with assert_fully_checked(self.device) as device:

            assert device.model == ModelEnum.PET_AIR_PRO

            assert device.pm1 is None
            assert device.pm2_5 is None
            assert device.pm10 is None
            assert device.total_voc is NotImplemented
            assert device.voc is None
            assert device.temperature is NotImplemented
            assert device.humidity is NotImplemented
            assert device.name == "PetAir Pro"
            assert device.firmware == "1.0.3"
            assert device.mcu_firmware == "1.0.3"
            assert device.serial_number == "111279300002313310000090"
            assert device.sku == "112793"

            assert device.standby is False
            assert device.night_mode is NotImplemented
            assert device.germ_shield is NotImplemented
            assert device.brightness == 100.0
            assert device.child_lock is False
            assert device.fan_speed == 91.0
            assert device.fan_auto_mode is NotImplemented
            assert device.filter_usage_percentage == 5.0
            assert device.wifi_working is False
            assert device.wick_usage_percentage is NotImplemented
            assert device.auto_regulated_humidity is NotImplemented
            assert device.water_shortage is NotImplemented
            assert device.wick_dry_mode is NotImplemented
            assert device.main_mode == 0.0
            assert device.heat_temp is NotImplemented
            assert device.heat_sub_mode is NotImplemented
            assert device.heat_fan_speed is NotImplemented
            assert device.cool_sub_mode is NotImplemented
            assert device.cool_fan_speed is NotImplemented
            assert device.ap_sub_mode == 1.0
            assert device.fan_speed_0 is None
            assert device.temperature_unit is NotImplemented
